@@ -1,16 +1,16 @@
 #' Convert .mp3 files to .wav
 #' 
 #' \code{mp32wav} converts several .mp3 files in working directory to .wav format
-#' @usage mp32wav()  
+#' @usage mp32wav(samp.rate = 44.1)  
+#' @param samp.rate Sampling rate at which the .wav files should be written. The maximum permitted is 44.1 kHz (default). Units should be kHz.
 #' @return .wav files saved in the working directory with same name as original mp3 files.
 #' @export
 #' @name mp32wav
 #' @examples
 #' \dontrun{
-#' # First create empty folder
-#' dir.create(file.path(getwd(),"temp"))
-#' setwd(file.path(getwd(),"temp"))
-#' 
+#' # First set temporary folder
+#' setwd(tempdir())
+#'  
 #' #Then download mp3 files from xeno-canto
 #' querxc(qword = "Phaethornis aethopygus", download = TRUE)
 #' 
@@ -19,19 +19,19 @@
 #' 
 #' #check this folder!!
 #' getwd()
-#' 
-#' #remove example directory
-#' unlink(getwd(),recursive = TRUE)
 #' }
 #' @details convert all .mp3 files in working directory to .wav format. Function used internally to read .mp3 files (\code{\link[tuneR]{readMP3}}) sometimes crashes
 #' This should be fixed in the next version of tuneR. 
 #' @author Marcelo Araya-Salas (\url{http://marceloarayasalas.weebly.com/}) and Grace Smith Vidaurre
 
-mp32wav <- function() {
-  options( show.error.messages = FALSE)  
+
+mp32wav <- function(samp.rate = 44.1) {
+  
+  if(samp.rate > 44.1) samp.rate <- 44.1
+  
   files <- list.files(path=getwd(), pattern = "mp3$", ignore.case = TRUE) #list .mp3 files in working directory
   if(length(files) == 0) stop("no 'wav' files in working directory")
   message("Start writing wav files:")
-  invisible(pblapply(files, function(x) tuneR::writeWave(tuneR::readMP3(filename = paste(getwd(), "/", x, sep="")),
-                                                paste(getwd(), "/", substr(x, 0, nchar(x) - 4), ".wav", sep=""))))
-}
+  a<-pbapply::pblapply(files, function(x) tuneR::writeWave(downsample(tuneR::readMP3(filename =  x), samp.rate = samp.rate * 1000), paste(substr(x, 0, nchar(x) - 4), ".wav", sep="")))
+return()  
+  }
\ No newline at end of file
