@@ -6,8 +6,8 @@
 #'   reverse.gray.colors.2, ovlp = 70, inner.mar = c(5, 4, 4, 2), outer.mar = 
 #'   c(0, 0, 0, 0), picsize = 1, res = 100, cexlab = 1, title = TRUE, propwidth = FALSE, 
 #'   xl = 1, osci = FALSE, gr = FALSE, sc = FALSE, bp = c(0, 22), cex = c(0.8, 1), 
-#'   threshold = 15, col = c("dodgerblue", "chartreuse3"), pch = c(16, 17),  mar = 0.05, 
-#'   lpos = "topright", it = "jpeg")
+#'   threshold = 15, col = c("chartreuse3", "dodgerblue"), pch = c(17, 16),  mar = 0.05, 
+#'   lpos = "topright", it = "jpeg", parallel = FALSE)
 #' @param  X Data frame with results containing columns for sound file name (sound.files), 
 #' selection number (selec), and start and end time of signal (start and end).
 #' The ouptut of \code{\link{manualoc}} or \code{\link{autodetec}} can be used as the input data frame. 
@@ -53,10 +53,10 @@
 #' @param threshold amplitude threshold (\%) for fundamental frequency and 
 #'   dominant frequency detection. Default is 15.
 #' @param col Vector of length 2 specifying colors of points plotted to mark 
-#'   fundamental and dominant frequency measurements. Default is c("dodgerblue",
-#'   "chartreuse3").
+#'   fundamental and dominant frequency measurements. Default is c("chartreuse3",
+#'   "dodgerblue").
 #' @param pch Numeric vector of length 2 specifying plotting characters for 
-#'   the frequency measurements. Default is c(16, 17).
+#'   the frequency measurements. Default is c(17, 16).
 #' @param mar Numeric vector of length 1. Specifies the margins adjacent to the selections
 #'  to set spectrogram limits. Default is 0.05.
 #' @param lpos Character vector of length 1 or numeric vector of length 2, 
@@ -66,6 +66,9 @@
 #'   Default is "topright".
 #' @param it A character vector of length 1 giving the image type to be used. Currently only
 #' "tiff" and "jpeg" are admitted. Default is "jpeg".
+#' @param parallel Either logical or numeric. Controls wehther parallel computing is applied.
+#'  If \code{TRUE} 2 cores are employed. If numeric, it specifies the number of cores to be used. 
+#'  Not available for windows OS. 
 #' @return Spectrograms of the signals listed in the input data frame showing the location of 
 #' the dominant and fundamental frequencies.
 #' @family spectrogram creators
@@ -115,8 +118,8 @@
 trackfreqs <- function(X, wl = 512, flim = c(0, 22), wn = "hanning", pal = reverse.gray.colors.2, ovlp = 70, 
                        inner.mar = c(5,4,4,2), outer.mar = c(0,0,0,0), picsize = 1, res = 100, cexlab = 1,
                        title = TRUE, propwidth = FALSE, xl = 1, osci = FALSE, gr = FALSE, sc = FALSE, 
-                       bp = c(0, 22), cex = c(0.8, 1), threshold = 15, col = c("dodgerblue", "chartreuse3"),
-                       pch = c(16, 17), mar = 0.05, lpos = "topright", it = "jpeg"){     
+                     bp = c(0, 22), cex = c(0.8, 1), threshold = 15, col = c("chartreuse3", "dodgerblue"),
+                       pch = c(17, 16), mar = 0.05, lpos = "topright", it = "jpeg", parallel = FALSE){     
 
   if(class(X) == "data.frame") {if(all(c("sound.files", "selec", 
                                          "start", "end") %in% colnames(X))) 
@@ -166,8 +169,14 @@ trackfreqs <- function(X, wl = 512, flim = c(0, 22), wn = "hanning", pal = rever
     sound.files <- sound.files[d]
   }
   
-  message("Creating spectrograms overlaid with acoustic measurements:")
-  invisible(pbapply::pbapply(matrix(c(1:length(sound.files)), ncol=1), 1, function(i){
+  #if parallel was called
+  if (parallel) {lapp <- function(X, FUN) parallel::mclapply(X, 
+    FUN, mc.cores = 2)} else    
+        if(is.numeric(parallel)) lapp <- function(X, FUN) parallel::mclapply(X, 
+              FUN, mc.cores = parallel) else lapp <- pbapply::pblapply
+  
+  if(!parallel) message("Creating spectrograms overlaid with acoustic measurements:")
+  invisible(lapp(1:length(sound.files), function(i){
     
     r <- tuneR::readWave(file.path(getwd(), sound.files[i]))
     
@@ -234,7 +243,7 @@ trackfreqs <- function(X, wl = 512, flim = c(0, 22), wn = "hanning", pal = rever
     dfreq <- seewave::dfreq(r, f = f, wl = wl, ovlp = 70, plot = FALSE, bandpass = b * 1000, fftw = TRUE, 
                    threshold = threshold, tlim = c(start[i], end[i]))
 
-    points(c(dfreq[,1])+start[i], c(dfreq[,2]), col = col[2], cex = cex[1], pch = pch[2]) 
+    points(dfreq[,1]+start[i], dfreq[,2], col = col[2], cex = cex[1], pch = pch[2]) 
     
     abline(v = c(end[i],start[i]), col= "red", lty = "dashed")
     
@@ -247,5 +256,5 @@ trackfreqs <- function(X, wl = 512, flim = c(0, 22), wn = "hanning", pal = rever
     return(NULL)
     
   }))
-message("all done!")
+
 }
